# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class TemplateKind(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    allowed enum values
    """
    BUCKET = "Bucket"
    CHECK = "Check"
    CHECKDEADMAN = "CheckDeadman"
    CHECKTHRESHOLD = "CheckThreshold"
    DASHBOARD = "Dashboard"
    LABEL = "Label"
    NOTIFICATIONENDPOINT = "NotificationEndpoint"
    NOTIFICATIONENDPOINTHTTP = "NotificationEndpointHTTP"
    NOTIFICATIONENDPOINTPAGERDUTY = "NotificationEndpointPagerDuty"
    NOTIFICATIONENDPOINTSLACK = "NotificationEndpointSlack"
    NOTIFICATIONRULE = "NotificationRule"
    TASK = "Task"
    TELEGRAF = "Telegraf"
    VARIABLE = "Variable"

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
    }

    attribute_map = {
    }

    def __init__(self):  # noqa: E501,D401,D403
        """TemplateKind - a model defined in OpenAPI."""  # noqa: E501        self.discriminator = None

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, TemplateKind):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



import random

import httpretty
import pytest
import reactivex as rx
import pandas
import warnings

from pandas import DataFrame
from pandas._libs.tslibs.timestamps import Timestamp
from reactivex import operators as ops

from influxdb_client import InfluxDBClient, Point, WritePrecision, WriteOptions
from influxdb_client.client.warnings import MissingPivotFunction
from influxdb_client.rest import ApiException
from tests.base_test import BaseTest, current_milli_time


class QueryDataFrameApi(BaseTest):

    def setUp(self) -> None:
        super(QueryDataFrameApi, self).setUp()

        httpretty.enable()
        httpretty.reset()

    def tearDown(self) -> None:
        self.client.close()
        httpretty.disable()

    def test_one_table(self):
        query_response = \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,long,string,string,string\n' \
            '#group,false,false,true,true,false,false,true,true,true\n' \
            '#default,_result,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_value,_field,_measurement,host\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:05Z,11125907456,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:06Z,11127103488,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:07Z,11127291904,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:08Z,11126190080,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:09Z,11127832576,used,mem,mac.local\n' \
            '\n\n'

        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body=query_response)

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        _dataFrame = self.client.query_api().query_data_frame(
            'from(bucket: "my-bucket") '
            '|> range(start: -5s, stop: now()) '
            '|> filter(fn: (r) => r._measurement == "mem") '
            '|> filter(fn: (r) => r._field == "used")',
            "my-org")

        self.assertEqual(DataFrame, type(_dataFrame))
        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_time", "_value", "_field", "_measurement", "host"],
            list(_dataFrame.columns))
        self.assertListEqual([0, 1, 2, 3, 4], list(_dataFrame.index))
        self.assertEqual(5, len(_dataFrame))
        self.assertEqual("_result", _dataFrame['result'][0])
        self.assertEqual("_result", _dataFrame['result'][1])
        self.assertEqual("_result", _dataFrame['result'][2])
        self.assertEqual("_result", _dataFrame['result'][3])
        self.assertEqual("_result", _dataFrame['result'][4])
        self.assertEqual(0, _dataFrame['table'][0], None)
        self.assertEqual(0, _dataFrame['table'][1], None)
        self.assertEqual(0, _dataFrame['table'][2], None)
        self.assertEqual(0, _dataFrame['table'][3], None)
        self.assertEqual(0, _dataFrame['table'][4], None)
        self.assertEqual(Timestamp('2019-11-12 08:09:04.795385+0000'), _dataFrame['_start'][0])
        self.assertEqual(Timestamp('2019-11-12 08:09:04.795385+0000'), _dataFrame['_start'][1])
        self.assertEqual(Timestamp('2019-11-12 08:09:04.795385+0000'), _dataFrame['_start'][2])
        self.assertEqual(Timestamp('2019-11-12 08:09:04.795385+0000'), _dataFrame['_start'][3])
        self.assertEqual(Timestamp('2019-11-12 08:09:04.795385+0000'), _dataFrame['_start'][4])
        self.assertEqual(Timestamp('2019-11-12 08:09:09.795385+0000'), _dataFrame['_stop'][0])
        self.assertEqual(Timestamp('2019-11-12 08:09:09.795385+0000'), _dataFrame['_stop'][1])
        self.assertEqual(Timestamp('2019-11-12 08:09:09.795385+0000'), _dataFrame['_stop'][2])
        self.assertEqual(Timestamp('2019-11-12 08:09:09.795385+0000'), _dataFrame['_stop'][3])
        self.assertEqual(Timestamp('2019-11-12 08:09:09.795385+0000'), _dataFrame['_stop'][4])
        self.assertEqual(Timestamp('2019-11-12 08:09:05+0000'), _dataFrame['_time'][0])
        self.assertEqual(Timestamp('2019-11-12 08:09:06+0000'), _dataFrame['_time'][1])
        self.assertEqual(Timestamp('2019-11-12 08:09:07+0000'), _dataFrame['_time'][2])
        self.assertEqual(Timestamp('2019-11-12 08:09:08+0000'), _dataFrame['_time'][3])
        self.assertEqual(Timestamp('2019-11-12 08:09:09+0000'), _dataFrame['_time'][4])
        self.assertEqual(11125907456, _dataFrame['_value'][0])
        self.assertEqual(11127103488, _dataFrame['_value'][1])
        self.assertEqual(11127291904, _dataFrame['_value'][2])
        self.assertEqual(11126190080, _dataFrame['_value'][3])
        self.assertEqual(11127832576, _dataFrame['_value'][4])
        self.assertEqual('used', _dataFrame['_field'][0])
        self.assertEqual('used', _dataFrame['_field'][1])
        self.assertEqual('used', _dataFrame['_field'][2])
        self.assertEqual('used', _dataFrame['_field'][3])
        self.assertEqual('used', _dataFrame['_field'][4])
        self.assertEqual('mem', _dataFrame['_measurement'][0])
        self.assertEqual('mem', _dataFrame['_measurement'][1])
        self.assertEqual('mem', _dataFrame['_measurement'][2])
        self.assertEqual('mem', _dataFrame['_measurement'][3])
        self.assertEqual('mem', _dataFrame['_measurement'][4])
        self.assertEqual('mac.local', _dataFrame['host'][0])
        self.assertEqual('mac.local', _dataFrame['host'][1])
        self.assertEqual('mac.local', _dataFrame['host'][2])
        self.assertEqual('mac.local', _dataFrame['host'][3])
        self.assertEqual('mac.local', _dataFrame['host'][4])

    def test_more_table(self):
        query_response = \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,long,string,string,string\n' \
            '#group,false,false,true,true,false,false,true,true,true\n' \
            '#default,_result,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_value,_field,_measurement,host\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:05Z,11125907456,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:06Z,11127103488,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:07Z,11127291904,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:08Z,11126190080,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:09Z,11127832576,used,mem,mac.local\n' \
            '\n\n' \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,long,string,string,string\n' \
            '#group,false,false,true,true,false,false,true,true,true\n' \
            '#default,_result,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_value,_field,_measurement,host\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:05Z,6053961728,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:06Z,6052765696,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:07Z,6052577280,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:08Z,6053679104,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:09Z,6052036608,available,mem,mac.local\n' \
            '\n\n' \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,long,string,string,string\n' \
            '#group,false,false,true,true,false,false,true,true,true\n' \
            '#default,_result,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_value,_field,_measurement,host\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:05Z,18632704,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:06Z,17420288,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:07Z,17256448,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:08Z,18362368,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:09Z,16723968,free,mem,mac.local\n\n'

        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body=query_response)

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        _dataFrames = self.client.query_api().query_data_frame(
            'from(bucket: "my-bucket") '
            '|> range(start: -5s, stop: now()) '
            '|> filter(fn: (r) => r._measurement == "mem") '
            '|> filter(fn: (r) => r._field == "available" or r._field == "free" or r._field == "used")',
            "my-org")

        self.assertEqual(list, type(_dataFrames))
        self.assertEqual(len(_dataFrames), 3)

        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_time", "_value", "_field", "_measurement", "host"],
            list(_dataFrames[0].columns))
        self.assertListEqual([0, 1, 2, 3, 4], list(_dataFrames[0].index))
        self.assertEqual(5, len(_dataFrames[0]))

        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_time", "_value", "_field", "_measurement", "host"],
            list(_dataFrames[1].columns))
        self.assertListEqual([0, 1, 2, 3, 4], list(_dataFrames[1].index))
        self.assertEqual(5, len(_dataFrames[1]))

        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_time", "_value", "_field", "_measurement", "host"],
            list(_dataFrames[2].columns))
        self.assertListEqual([0, 1, 2, 3, 4], list(_dataFrames[2].index))
        self.assertEqual(5, len(_dataFrames[2]))

    def test_empty_data_set(self):
        query_response = '\n'

        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body=query_response)

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        _dataFrame = self.client.query_api().query_data_frame(
            'from(bucket: "my-bucket") '
            '|> range(start: -5s, stop: now()) '
            '|> filter(fn: (r) => r._measurement == "mem") '
            '|> filter(fn: (r) => r._field == "not_exit")',
            "my-org")

        self.assertEqual(DataFrame, type(_dataFrame))
        self.assertListEqual([], list(_dataFrame.columns))
        self.assertListEqual([], list(_dataFrame.index))

    def test_more_table_custom_index(self):
        query_response = \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,long,string,string,string\n' \
            '#group,false,false,true,true,false,false,true,true,true\n' \
            '#default,_result,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_value,_field,_measurement,host\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:05Z,11125907456,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:06Z,11127103488,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:07Z,11127291904,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:08Z,11126190080,used,mem,mac.local\n' \
            ',,0,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:09Z,11127832576,used,mem,mac.local\n' \
            '\n\n' \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,long,string,string,string\n' \
            '#group,false,false,true,true,false,false,true,true,true\n' \
            '#default,_result,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_value,_field,_measurement,host\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:05Z,6053961728,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:06Z,6052765696,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:07Z,6052577280,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:08Z,6053679104,available,mem,mac.local\n' \
            ',,1,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:09Z,6052036608,available,mem,mac.local\n' \
            '\n\n' \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,long,string,string,string\n' \
            '#group,false,false,true,true,false,false,true,true,true\n' \
            '#default,_result,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_value,_field,_measurement,host\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:05Z,18632704,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:06Z,17420288,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:07Z,17256448,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:08Z,18362368,free,mem,mac.local\n' \
            ',,2,2019-11-12T08:09:04.795385031Z,2019-11-12T08:09:09.795385031Z,2019-11-12T08:09:09Z,16723968,free,mem,mac.local\n\n'

        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body=query_response)

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        _dataFrames = self.client.query_api().query_data_frame(
            'from(bucket: "my-bucket") '
            '|> range(start: -5s, stop: now()) '
            '|> filter(fn: (r) => r._measurement == "mem") '
            '|> filter(fn: (r) => r._field == "available" or r._field == "free" or r._field == "used")',
            "my-org", data_frame_index=["_time"])

        self.assertEqual(list, type(_dataFrames))
        self.assertEqual(len(_dataFrames), 3)

        print(_dataFrames[0].to_string())
        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_value", "_field", "_measurement", "host"],
            list(_dataFrames[0].columns))
        self.assertListEqual([Timestamp('2019-11-12 08:09:05+0000'), Timestamp('2019-11-12 08:09:06+0000'),
                              Timestamp('2019-11-12 08:09:07+0000'), Timestamp('2019-11-12 08:09:08+0000'),
                              Timestamp('2019-11-12 08:09:09+0000')], list(_dataFrames[0].index))
        self.assertEqual(5, len(_dataFrames[0]))

        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_value", "_field", "_measurement", "host"],
            list(_dataFrames[1].columns))
        self.assertListEqual([Timestamp('2019-11-12 08:09:05+0000'), Timestamp('2019-11-12 08:09:06+0000'),
                              Timestamp('2019-11-12 08:09:07+0000'), Timestamp('2019-11-12 08:09:08+0000'),
                              Timestamp('2019-11-12 08:09:09+0000')], list(_dataFrames[1].index))
        self.assertEqual(5, len(_dataFrames[1]))

        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_value", "_field", "_measurement", "host"],
            list(_dataFrames[2].columns))
        self.assertListEqual([Timestamp('2019-11-12 08:09:05+0000'), Timestamp('2019-11-12 08:09:06+0000'),
                              Timestamp('2019-11-12 08:09:07+0000'), Timestamp('2019-11-12 08:09:08+0000'),
                              Timestamp('2019-11-12 08:09:09+0000')], list(_dataFrames[2].index))
        self.assertEqual(5, len(_dataFrames[2]))

    def test_query_with_warning(self):
        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body='\n')

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        with pytest.warns(MissingPivotFunction) as warnings:
            self.client.query_api().query_data_frame(
                'from(bucket: "my-bucket")'
                '|> range(start: -5s, stop: now()) '
                '|> filter(fn: (r) => r._measurement == "mem") '
                "my-org")
        self.assertEqual(1, len([w for w in warnings if w.category == MissingPivotFunction]))

    def test_query_without_warning(self):
        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body='\n')

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        with warnings.catch_warnings(record=True) as warns:
            self.client.query_api().query_data_frame(
                'import "influxdata/influxdb/schema"'
                ''
                'from(bucket: "my-bucket")'
                '|> range(start: -5s, stop: now()) '
                '|> filter(fn: (r) => r._measurement == "mem") '
                '|> schema.fieldsAsCols() '
                "my-org")
        self.assertEqual(0, len([w for w in warns if w.category == MissingPivotFunction]))

        with warnings.catch_warnings(record=True) as warns:
            self.client.query_api().query_data_frame(
                'from(bucket: "my-bucket")'
                '|> range(start: -5s, stop: now()) '
                '|> filter(fn: (r) => r._measurement == "mem") '
                '|> pivot(rowKey: ["_time"], columnKey: ["_field"], valueColumn: "_value")'
                "my-org")
        self.assertEqual(0, len([w for w in warns if w.category == MissingPivotFunction]))

    def test_pivoted_data(self):
        query_response = \
            '#group,false,false,true,true,false,true,false,false,false,false\n' \
            '#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,string,double,long,string,boolean\n' \
            '#default,_result,,,,,,,,,\n' \
            ',result,table,_start,_stop,_time,_measurement,test_double,test_long,test_string,test_boolean\n' \
            ',,0,2023-12-15T13:19:45Z,2023-12-15T13:20:00Z,2023-12-15T13:19:55Z,test,4,,,\n' \
            ',,0,2023-12-15T13:19:45Z,2023-12-15T13:20:00Z,2023-12-15T13:19:56Z,test,,1,,\n' \
            ',,0,2023-12-15T13:19:45Z,2023-12-15T13:20:00Z,2023-12-15T13:19:57Z,test,,,hi,\n' \
            ',,0,2023-12-15T13:19:45Z,2023-12-15T13:20:00Z,2023-12-15T13:19:58Z,test,,,,true\n' \
            '\n\n'

        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body=query_response)

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        _dataFrame = self.client.query_api().query_data_frame(
            'from(bucket: "my-bucket") '
            '|> range(start: 2023-12-15T13:19:45Z, stop: 2023-12-15T13:20:00Z)'
            '|> filter(fn: (r) => r["_measurement"] == "test")'
            '|> pivot(rowKey:["_time"], columnKey: ["_field"], valueColumn: "_value")'
            "my-org", use_extension_dtypes=True)

        self.assertEqual(DataFrame, type(_dataFrame))
        self.assertListEqual(
            ["result", "table", "_start", "_stop", "_time", "_measurement",
             "test_double", "test_long", "test_string", "test_boolean"],
            list(_dataFrame.columns))
        self.assertListEqual([0, 1, 2, 3], list(_dataFrame.index))
        # self.assertEqual('Int64', _dataFrame.dtypes['test_long'].name)
        # self.assertEqual('Float64', _dataFrame.dtypes['test_double'].name)
        self.assertEqual('string', _dataFrame.dtypes['test_string'].name)
        self.assertEqual('boolean', _dataFrame.dtypes['test_boolean'].name)
        self.assertEqual(4, len(_dataFrame))
        self.assertEqual("_result", _dataFrame['result'][0])
        self.assertEqual("_result", _dataFrame['result'][1])
        self.assertEqual("_result", _dataFrame['result'][2])
        self.assertEqual("_result", _dataFrame['result'][3])
        self.assertEqual(0, _dataFrame['table'][0], None)
        self.assertEqual(0, _dataFrame['table'][1], None)
        self.assertEqual(0, _dataFrame['table'][2], None)
        self.assertEqual(0, _dataFrame['table'][3], None)
        self.assertEqual(Timestamp('2023-12-15 13:19:45+0000'), _dataFrame['_start'][0])
        self.assertEqual(Timestamp('2023-12-15 13:19:45+0000'), _dataFrame['_start'][1])
        self.assertEqual(Timestamp('2023-12-15 13:19:45+0000'), _dataFrame['_start'][2])
        self.assertEqual(Timestamp('2023-12-15 13:19:45+0000'), _dataFrame['_start'][3])
        self.assertEqual(Timestamp('2023-12-15 13:20:00+0000'), _dataFrame['_stop'][0])
        self.assertEqual(Timestamp('2023-12-15 13:20:00+0000'), _dataFrame['_stop'][1])
        self.assertEqual(Timestamp('2023-12-15 13:20:00+0000'), _dataFrame['_stop'][2])
        self.assertEqual(Timestamp('2023-12-15 13:20:00+0000'), _dataFrame['_stop'][3])
        self.assertEqual(Timestamp('2023-12-15 13:19:55+0000'), _dataFrame['_time'][0])
        self.assertEqual(Timestamp('2023-12-15 13:19:56+0000'), _dataFrame['_time'][1])
        self.assertEqual(Timestamp('2023-12-15 13:19:57+0000'), _dataFrame['_time'][2])
        self.assertEqual(Timestamp('2023-12-15 13:19:58+0000'), _dataFrame['_time'][3])
        self.assertEqual(4, _dataFrame['test_double'][0])
        self.assertTrue(pandas.isna(_dataFrame['test_double'][1]))
        self.assertTrue(pandas.isna(_dataFrame['test_double'][2]))
        self.assertTrue(pandas.isna(_dataFrame['test_double'][3]))
        self.assertTrue(pandas.isna(_dataFrame['test_long'][0]))
        self.assertEqual(1, _dataFrame['test_long'][1])
        self.assertTrue(pandas.isna(_dataFrame['test_long'][2]))
        self.assertTrue(pandas.isna(_dataFrame['test_long'][3]))
        self.assertTrue(pandas.isna(_dataFrame['test_string'][0]))
        self.assertTrue(pandas.isna(_dataFrame['test_string'][1]))
        self.assertEqual('hi', _dataFrame['test_string'][2])
        self.assertTrue(pandas.isna(_dataFrame['test_string'][3]))
        self.assertTrue(pandas.isna(_dataFrame['test_boolean'][0]))
        self.assertTrue(pandas.isna(_dataFrame['test_boolean'][1]))
        self.assertTrue(pandas.isna(_dataFrame['test_boolean'][2]))
        self.assertEqual(True, _dataFrame['test_boolean'][3])


class QueryDataFrameIntegrationApi(BaseTest):

    def test_large_amount_of_data(self):
        _measurement_name = "data_frame_" + str(current_milli_time())

        def _create_point(index) -> Point:
            return Point(_measurement_name) \
                .tag("deviceType", str(random.choice(['A', 'B']))) \
                .tag("name", random.choice(['A', 'B'])) \
                .field("uuid", random.randint(0, 10_000)) \
                .field("co2", random.randint(0, 10_000)) \
                .field("humid", random.randint(0, 10_000)) \
                .field("lux", random.randint(0, 10_000)) \
                .field("water", random.randint(0, 10_000)) \
                .field("shine", random.randint(0, 10_000)) \
                .field("temp", random.randint(0, 10_000)) \
                .field("voc", random.randint(0, 10_000)) \
                .time(time=(1583828781 + index), write_precision=WritePrecision.S)

        data = rx.range(0, 2_000).pipe(ops.map(lambda index: _create_point(index)))

        write_api = self.client.write_api(write_options=WriteOptions(batch_size=500))
        write_api.write(org="my-org", bucket="my-bucket", record=data, write_precision=WritePrecision.S)
        write_api.close()

        query = 'from(bucket: "my-bucket")' \
                '|> range(start: 2020-02-19T23:30:00Z, stop: now())' \
                f'|> filter(fn: (r) => r._measurement == "{_measurement_name}")'

        result = self.client.query_api().query_data_frame(org="my-org", query=query)

        self.assertGreater(len(result), 1)

    def test_query_without_credentials(self):
        _client = InfluxDBClient(url="http://localhost:8086", token="my-token-wrong-credentials", org="my-org")

        with self.assertRaises(ApiException) as ae:
            query = 'from(bucket: "my-bucket")' \
                    '|> range(start: 2020-02-19T23:30:00Z, stop: now())' \
                    f'|> filter(fn: (r) => r._measurement == "my-measurement")'
            _client.query_api().query_data_frame(query=query)

        exception = ae.exception
        self.assertEqual(401, exception.status)
        self.assertEqual("Unauthorized", exception.reason)

        _client.close()



import pytest

from influxdb_client import InfluxDBClient, Organization, PermissionResource, Permission
# noinspection PyProtectedMember
from influxdb_client.client.exceptions import InfluxDBError
from influxdb_client.client.util.helpers import get_org_query_param, _is_id
from tests.base_test import BaseTest


class HelpersTest(BaseTest):

    def test_is_id(self):
        self.assertTrue(_is_id("ffffffffffffffff"))
        self.assertTrue(_is_id("020f755c3c082000"))
        self.assertTrue(_is_id("ca55e77eca55e77e"))
        self.assertTrue(_is_id("02def021097c6000"))
        self.assertFalse(_is_id("gggggggggggggggg"))
        self.assertFalse(_is_id("abc"))
        self.assertFalse(_is_id("abcdabcdabcdabcd0"))
        self.assertFalse(_is_id("020f75"))
        self.assertFalse(_is_id("020f755c3c082000aaa"))
        self.assertFalse(_is_id(None))

    def test_organization_as_query_param(self):
        organization = Organization(id="org-id", name="org-name")
        org = get_org_query_param(organization, self.client)
        self.assertEqual("org-id", org)

    def test_required_id(self):
        org = get_org_query_param(None, self.client, required_id=True)
        self.assertEqual(self.my_organization.id, org)

    def test_required_id_not_exist(self):
        with pytest.raises(InfluxDBError) as e:
            get_org_query_param("not_exist_name", self.client, required_id=True)
        assert "The client cannot find organization with name: 'not_exist_name' to determine their ID." in f"{e.value} "

    def test_both_none(self):
        self.client.close()
        self.client = InfluxDBClient(url=self.client.url, token="my-token")
        org = get_org_query_param(None, self.client)
        self.assertIsNone(org)

    def test_not_permission_to_read_org(self):
        # Create Token without permission to read Organizations
        resource = PermissionResource(type="buckets", org_id=self.find_my_org().id)
        authorization = self.client \
            .authorizations_api() \
            .create_authorization(org_id=self.find_my_org().id,
                                  permissions=[Permission(resource=resource, action="read"),
                                               Permission(resource=resource, action="write")])
        self.client.close()

        # Initialize client without permission to read Organizations
        self.client = InfluxDBClient(url=self.client.url, token=authorization.token)

        with pytest.raises(InfluxDBError) as e:
            get_org_query_param("my-org", self.client, required_id=True)
        assert "The client cannot find organization with name: 'my-org' to determine their ID. Are you using token " \
               "with sufficient permission?" in f"{e.value} "



import datetime
import json
import unittest

from datetime import timezone
from httpretty import httpretty

from influxdb_client import QueryApi, DurationLiteral, Duration, CallExpression, UnaryExpression, \
    Identifier, InfluxDBClient
from influxdb_client.client.query_api import QueryOptions
from influxdb_client.client.util.date_utils import get_date_helper
from tests.base_test import BaseTest


class SimpleQueryTest(BaseTest):

    def setUp(self) -> None:
        super(SimpleQueryTest, self).setUp()

        httpretty.enable()
        httpretty.reset()

    def tearDown(self) -> None:
        self.client.close()
        httpretty.disable()

    def test_query_raw(self):
        client = self.client

        query_client = client.query_api()

        bucket = "my-bucket"
        result = query_client.query_raw(
            'from(bucket:"' + bucket + '") |> range(start: 1970-01-01T00:00:00.000000001Z) |> last()')

        for x in result:
            print(x, end='')

    def test_query_flux_table(self):
        client = self.client
        bucket = "my-bucket"

        query_client = client.query_api()
        tables = query_client.query(
            'from(bucket:"' + bucket + '") |> range(start: 1970-01-01T00:00:00.000000001Z) |> last()')

        val_count = 0
        for table in tables:
            for row in table:
                for _ in row.values:
                    val_count += 1

        print("Values count: ", val_count)

    def test_query_flux_csv(self):
        client = self.client
        bucket = "my-bucket"
        query_client = client.query_api()
        csv_result = query_client.query_csv(
            'from(bucket:"' + bucket + '") |> range(start: 1970-01-01T00:00:00.000000001Z) |> last()')

        val_count = 0
        for row in csv_result:
            for _ in row:
                val_count += 1

        print("Values count: ", val_count)

    def test_query_ast(self):
        q = '''
        from(bucket:stringParam) 
            |> range(start: startDuration, stop: callParam)
            |> filter(fn: (r) => r["_measurement"] == "my_measurement")
            |> filter(fn: (r) => r["_value"] > intParam)
            |> filter(fn: (r) => r["_value"] > floatParam)
            |> aggregateWindow(every: durationParam, fn: mean, createEmpty: true)
            |> sort(columns: ["_time"], desc: booleanParam) 
        '''

        p = {
            "stringParam": "my-bucket",
            "stopParam": get_date_helper().parse_date("2021-03-20T15:59:10.607352Z"),
            "intParam": 2,
            "durationParam": DurationLiteral("DurationLiteral", [Duration(magnitude=1, unit="d")]),
            "startDuration": UnaryExpression(type="UnaryExpression",
                                             argument=DurationLiteral("DurationLiteral",
                                                                      [Duration(magnitude=30, unit="d")]),
                                             operator="-"),
            "callParam": CallExpression(type="CallExpression", callee=Identifier(type="Identifier", name="now")),
            "timedelta": datetime.timedelta(minutes=10),
            "floatParam": 14.01,
            "booleanParam": True,
        }

        csv_result = self.client.query_api().query_csv(query=q, params=p)

        self.assertIsNotNone(csv_result)

        val_count = 0
        for row in csv_result:
            for _ in row:
                val_count += 1

        print("Values count: ", val_count)

    def test_parameter_ast(self):
        test_data = [["stringParam", "my-bucket", {
            "imports": [],
            "body": [
                {
                    "type": "OptionStatement",
                    "assignment": {
                        "type": "VariableAssignment",
                        "id": {
                            "type": "Identifier",
                            "name": "stringParam"
                        },
                        "init": {
                            "type": "StringLiteral",
                            "value": "my-bucket"
                        }
                    }
                }
            ]
        }], ["datetimeParam", get_date_helper().parse_date("2021-03-20T15:59:10.607352Z"), {
            "body": [
                {
                    "assignment": {
                        "id": {
                            "name": "datetimeParam",
                            "type": "Identifier"
                        },
                        "init": {
                            "type": "DateTimeLiteral",
                            "value": "2021-03-20T15:59:10.607352000Z"
                        },
                        "type": "VariableAssignment"
                    },
                    "type": "OptionStatement"
                }
            ],
            "imports": []
        }], ["datetimeNoTZParam", datetime.datetime(2021, 3, 20, 15, 59, 10, 607352), {
            "body": [
                {
                    "assignment": {
                        "id": {
                            "name": "datetimeNoTZParam",
                            "type": "Identifier"
                        },
                        "init": {
                            "type": "DateTimeLiteral",
                            "value": "2021-03-20T15:59:10.607352000Z"
                        },
                        "type": "VariableAssignment"
                    },
                    "type": "OptionStatement"
                }
            ],
            "imports": []
        }], ["timeDeltaParam", datetime.timedelta(hours=1), {
            "body": [
                {
                    "assignment": {
                        "id": {
                            "name": "timeDeltaParam",
                            "type": "Identifier"
                        },
                        "init": {
                            "type": "DurationLiteral",
                            "values": [
                                {
                                    "magnitude": 3600000000,
                                    "unit": "us"
                                }
                            ]
                        },
                        "type": "VariableAssignment"
                    },
                    "type": "OptionStatement"
                }
            ],
            "imports": []
        }], ["timeDeltaNegativeParam", datetime.timedelta(minutes=-5), {
            "body": [
                {
                    "assignment": {
                        "id": {
                            "name": "timeDeltaNegativeParam",
                            "type": "Identifier"
                        },
                        "init": {
                            "argument": {
                                "type": "DurationLiteral",
                                "values": [
                                    {
                                        "magnitude": 300000000,
                                        "unit": "us"
                                    }
                                ]
                            },
                            "operator": "-",
                            "type": "UnaryExpression"
                        },
                        "type": "VariableAssignment"
                    },
                    "type": "OptionStatement"
                }
            ],
            "imports": []
        }], ["booleanParam", True, {
            "body": [
                {
                    "assignment": {
                        "id": {
                            "name": "booleanParam",
                            "type": "Identifier"
                        },
                        "init": {
                            "type": "BooleanLiteral",
                            "value": True
                        },
                        "type": "VariableAssignment"
                    },
                    "type": "OptionStatement"
                }
            ],
            "imports": []
        }], ["intParam", int(10), {
            "body": [
                {
                    "assignment": {
                        "id": {
                            "name": "intParam",
                            "type": "Identifier"
                        },
                        "init": {
                            "type": "IntegerLiteral",
                            "value": "10"
                        },
                        "type": "VariableAssignment"
                    },
                    "type": "OptionStatement"
                }
            ],
            "imports": []
        }], ["floatParam", 10.333,
             {
                 "body": [
                     {
                         "assignment": {
                             "id": {
                                 "name": "floatParam",
                                 "type": "Identifier"
                             },
                             "init": {
                                 "type": "FloatLiteral",
                                 "value": 10.333
                             },
                             "type": "VariableAssignment"
                         },
                         "type": "OptionStatement"
                     }
                 ],
                 "imports": []
             }], ["arrayParam", ["bar1", "bar2", "bar3"],
                  {
                      "body": [
                          {
                              "assignment": {
                                  "id": {
                                      "name": "arrayParam",
                                      "type": "Identifier"
                                  },
                                  "init": {
                                      "elements": [
                                          {
                                              "type": "StringLiteral",
                                              "value": "bar1"
                                          },
                                          {
                                              "type": "StringLiteral",
                                              "value": "bar2"
                                          },
                                          {
                                              "type": "StringLiteral",
                                              "value": "bar3"
                                          }
                                      ],
                                      "type": "ArrayExpression"
                                  },
                                  "type": "VariableAssignment"
                              },
                              "type": "OptionStatement"
                          }
                      ],
                      "imports": []
                  }]]

        for data in test_data:
            param = {data[0]: data[1]}
            print("testing: ", param)
            ast = QueryApi._build_flux_ast(param)
            got_sanitized = self.client.api_client.sanitize_for_serialization(ast)
            self.assertEqual(json.dumps(got_sanitized, sort_keys=True, indent=2),
                             json.dumps(data[2], sort_keys=True, indent=2))

    def test_query_profiler_enabled(self):
        q = '''
        from(bucket:stringParam) 
            |> range(start: 0, stop: callParam) |> last()
        '''
        p = {
            "stringParam": "my-bucket",
            "stopParam": get_date_helper().parse_date("2021-03-20T15:59:10.607352Z"),
            "durationParam": DurationLiteral("DurationLiteral", [Duration(magnitude=1, unit="d")]),
            "callParam": CallExpression(type="CallExpression", callee=Identifier(type="Identifier", name="now")),
        }
        query_api = self.client.query_api(query_options=QueryOptions(profilers=["query", "operator"]))
        csv_result = query_api.query(query=q, params=p)

        for table in csv_result:
            self.assertFalse(any(filter(lambda column: (column.default_value == "_profiler"), table.columns)))
            for flux_record in table:
                self.assertFalse(flux_record["_measurement"].startswith("profiler/"))

        records = self.client.query_api().query_stream(query=q, params=p)

        for flux_record in records:
            self.assertFalse(flux_record["_measurement"].startswith("profiler/"))

        self.assertIsNotNone(csv_result)

    def test_query_profiler_present(self):

        client = self.client
        q = '''
        import "profiler"

        option profiler.enabledProfilers = ["query", "operator"]

        from(bucket:stringParam) 
            |> range(start: 0, stop: callParam) |> last()
        '''

        p = {
            "stringParam": "my-bucket",
            "stopParam": get_date_helper().parse_date("2021-03-20T15:59:10.607352Z"),
            "durationParam": DurationLiteral("DurationLiteral", [Duration(magnitude=1, unit="d")]),
            "callParam": CallExpression(type="CallExpression", callee=Identifier(type="Identifier", name="now")),
        }
        csv_result = client.query_api(query_options=QueryOptions(profilers=None)).query(query=q, params=p)
        self.assertIsNotNone(csv_result)

        found_profiler_table = False
        found_profiler_records = False

        for table in csv_result:
            if any(filter(lambda column: (column.default_value == "_profiler"), table.columns)):
                found_profiler_table = True
                print(f"Profiler table : {table} ")
            for flux_record in table:
                if flux_record["_measurement"].startswith("profiler/"):
                    found_profiler_records = True
                    print(f"Profiler record: {flux_record}")

        self.assertTrue(found_profiler_table)
        self.assertTrue(found_profiler_records)

        records = client.query_api().query_stream(query=q, params=p)

        found_profiler_records = False
        for flux_record in records:
            if flux_record["_measurement"].startswith("profiler/"):
                found_profiler_records = True
                print(f"Profiler record: {flux_record}")
        self.assertTrue(found_profiler_records)

    def test_profilers_callback(self):

        class ProfilersCallback(object):
            def __init__(self):
                self.records = []

            def __call__(self, flux_record):
                self.records.append(flux_record.values)

            def get_record(self, num, val):
                return (self.records[num])[val]

        callback = ProfilersCallback()

        query_api = self.client.query_api(query_options=QueryOptions(profilers=["query", "operator"],
                                                                     profiler_callback=callback))
        query_api.query('from(bucket:"my-bucket") |> range(start: -10m)')

        self.assertEqual("profiler/query", callback.get_record(0, "_measurement"))
        self.assertEqual("profiler/operator", callback.get_record(1, "_measurement"))

    def test_profiler_ast(self):

        expect = {
            "body": [
                {
                    "assignment": {
                        "init": {
                            "elements": [
                                {
                                    "type": "StringLiteral",
                                    "value": "first-profiler"
                                },
                                {
                                    "type": "StringLiteral",
                                    "value": "second-profiler"
                                }
                            ],
                            "type": "ArrayExpression"
                        },
                        "member": {
                            "object": {
                                "name": "profiler",
                                "type": "Identifier"
                            },
                            "property": {
                                "name": "enabledProfilers",
                                "type": "Identifier"
                            },
                            "type": "MemberExpression"
                        },
                        "type": "MemberAssignment"
                    },
                    "type": "OptionStatement"
                }
            ],
            "imports": [
                {
                    "path": {
                        "type": "StringLiteral",
                        "value": "profiler"
                    },
                    "type": "ImportDeclaration"
                }
            ]
        }

        ast = QueryApi._build_flux_ast(params=None, profilers=["first-profiler", "second-profiler"])
        got_sanitized = self.client.api_client.sanitize_for_serialization(ast)
        print(json.dumps(got_sanitized, sort_keys=True, indent=2))

        self.assertEqual(json.dumps(got_sanitized, sort_keys=True, indent=2),
                         json.dumps(expect, sort_keys=True, indent=2))

    def test_profiler_mock(self):

        query_response = """#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,string,string,double,double,double
#group,false,false,true,true,false,true,true,false,false,false
#default,_result,,,,,,,,,
,result,table,_start,_stop,_time,_measurement,host,available,free,used
,,0,2021-05-24T08:40:44.7850004Z,2021-05-24T08:45:44.7850004Z,2021-05-24T08:41:00Z,mem,kozel.local,5832097792,317063168,11347771392
,,0,2021-05-24T08:40:44.7850004Z,2021-05-24T08:45:44.7850004Z,2021-05-24T08:42:00Z,mem,kozel.local,5713765717.333333,118702080,11466103466.666666
,,0,2021-05-24T08:40:44.7850004Z,2021-05-24T08:45:44.7850004Z,2021-05-24T08:43:00Z,mem,kozel.local,5776302080,135763968,11403567104
,,0,2021-05-24T08:40:44.7850004Z,2021-05-24T08:45:44.7850004Z,2021-05-24T08:44:00Z,mem,kozel.local,5758485162.666667,85798229.33333333,11421384021.333334
,,0,2021-05-24T08:40:44.7850004Z,2021-05-24T08:45:44.7850004Z,2021-05-24T08:45:00Z,mem,kozel.local,5788656981.333333,119243434.66666667,11391212202.666666
,,0,2021-05-24T08:40:44.7850004Z,2021-05-24T08:45:44.7850004Z,2021-05-24T08:45:44.7850004Z,mem,kozel.local,5727718400,35330048,11452150784

#datatype,string,long,string,long,long,long,long,long,long,long,long,long,string,string,long,long
#group,false,false,true,false,false,false,false,false,false,false,false,false,false,false,false,false
#default,_profiler,,,,,,,,,,,,,,,
,result,table,_measurement,TotalDuration,CompileDuration,QueueDuration,PlanDuration,RequeueDuration,ExecuteDuration,Concurrency,MaxAllocated,TotalAllocated,RuntimeErrors,flux/query-plan,influxdb/scanned-bytes,influxdb/scanned-values
,,0,profiler/query,8924700,350900,33800,0,0,8486500,0,2072,0,,"digraph {
  ReadWindowAggregateByTime11
  // every = 1m, aggregates = [mean], createEmpty = true, timeColumn = ""_stop""
  pivot8
  generated_yield

  ReadWindowAggregateByTime11 -> pivot8
  pivot8 -> generated_yield
}

",0,0

#datatype,string,long,string,string,string,long,long,long,long,double
#group,false,false,true,false,false,false,false,false,false,false
#default,_profiler,,,,,,,,,
,result,table,_measurement,Type,Label,Count,MinDuration,MaxDuration,DurationSum,MeanDuration
,,1,profiler/operator,*universe.pivotTransformation,pivot8,3,32600,126200,193400,64466.666666666664
,,1,profiler/operator,*influxdb.readWindowAggregateSource,ReadWindowAggregateByTime11,1,940500,940500,940500,940500
"""

        query = """
        from(bucket: "my-bucket") 
          |> range(start: -5m, stop: now()) 
          |> filter(fn: (r) => r._measurement == "mem") 
          |> filter(fn: (r) => r._field == "available" or r._field == "free" or r._field == "used")
          |> aggregateWindow(every: 1m, fn: mean)
          |> pivot(rowKey:["_time"], columnKey: ["_field"], valueColumn: "_value")
        """

        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body=query_response)
        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)
        query_api = self.client.query_api(query_options=QueryOptions(profilers=["query", "operator"]))
        tables = query_api.query(query=query)
        self.assertEqual(len(tables), 1)
        self.assertEqual(len(tables[0].columns), 10)
        self.assertEqual(len(tables[0].records), 6)

        self.assertEqual(tables[0].records[5].values,
                          {'result': '_result', 'table': 0,
                           '_start': datetime.datetime(2021, 5, 24, 8, 40, 44, 785000, tzinfo=timezone.utc),
                           '_stop': datetime.datetime(2021, 5, 24, 8, 45, 44, 785000, tzinfo=timezone.utc),
                           '_time': datetime.datetime(2021, 5, 24, 8, 45, 44, 785000, tzinfo=timezone.utc),
                           '_measurement': 'mem',
                           'host': 'kozel.local',
                           'available': 5727718400, 'free': 35330048,
                           'used': 11452150784})

    def test_time_to_ast(self):
        from influxdb_client.extras import pd
        import dateutil.parser

        literals = [
            (pd.Timestamp('1996-02-25T21:20:00.001001231Z'), '1996-02-25T21:20:00.001001231Z'),
            (dateutil.parser.parse('1996-02-25T21:20:00.001001231Z'), '1996-02-25T21:20:00.001001000Z'),
            (dateutil.parser.parse('1996-02-25'), '1996-02-25T00:00:00.000000000Z'),
            (datetime.datetime(2021, 5, 24, 8, 40, 44, 785000, tzinfo=timezone.utc), '2021-05-24T08:40:44.785000000Z'),
        ]

        for literal in literals:
            ast = QueryApi._build_flux_ast({'date': literal[0]})
            self.assertEqual('DateTimeLiteral', ast.body[0].assignment.init.type)
            self.assertEqual(literal[1], ast.body[0].assignment.init.value)

    def test_csv_empty_lines(self):
        query_response = '#datatype,string,long,dateTime:RFC3339,double,string\n' \
                         '#group,false,false,false,false,true\n' \
                         '#default,_result,,,,\n' \
                         ',result,table,_time,_value,_field\n' \
                         ',,0,2022-11-24T10:00:10Z,0.1,_1_current_(mA)\n' \
                         ',,1,2022-11-24T10:00:10Z,4,_1_current_limit_(mA)\n' \
                         ',,2,2022-11-24T10:00:10Z,1,_1_voltage_(V)\n' \
                         ',,3,2022-11-24T10:00:10Z,1,_1_voltage_limit_(V)\n' \
                         ',,4,2022-11-24T10:00:10Z,0,_2_current_(mA)\n' \
                         ',,5,2022-11-24T10:00:10Z,0,_2_current_limit_(mA)\n' \
                         ',,6,2022-11-24T10:00:10Z,0,_2_voltage_(V)\n' \
                         ',,7,2022-11-24T10:00:10Z,0,_2_voltage_limit_(V)\n' \
                         '\n' \
                         '\n' \
                         '#datatype,string,long,dateTime:RFC3339,string,string\n' \
                         '#group,false,false,false,false,true\n' \
                         '#default,_result,,,,\n' \
                         ',result,table,_time,_value,_field\n' \
                         ',,8,2022-11-24T10:00:10Z,K,type\n' \
                         ',,9,2022-11-24T10:00:10Z,,type2\n' \
                         '\n'

        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/query", status=200, body=query_response)

        self.client = InfluxDBClient("http://localhost", "my-token", org="my-org", enable_gzip=False)

        csv_lines = list(self.client.query_api().query_csv('from(bucket: "my-bucket")', "my-org"))
        self.assertEqual(18, len(csv_lines))
        for csv_line in csv_lines:
            self.assertEqual(6, len(csv_line))

        # to_values
        csv_lines = self.client.query_api().query_csv('from(bucket: "my-bucket")', "my-org").to_values()
        self.assertEqual(18, len(csv_lines))

if __name__ == '__main__':
    unittest.main()



from tests.base_test import BaseTest, generate_name


class OrganizationsApiTests(BaseTest):

    def setUp(self) -> None:
        super(OrganizationsApiTests, self).setUp()
        organizations_api = self.client.organizations_api()
        organizations = organizations_api.find_organizations()

        for organization in organizations:
            if organization.name.endswith("_IT"):
                print("Delete organization: ", organization.name)
                organizations_api.delete_organization(org_id=organization.id)

    def test_update_organization(self):
        organizations_api = self.client.organizations_api()

        organization = organizations_api.create_organization(name=generate_name(key='org'))
        self.assertEqual("", organization.description)

        organization.description = "updated description"
        organization = organizations_api.update_organization(organization=organization)
        self.assertEqual("updated description", organization.description)



"""
An organization is a workspace for a group of users.

All dashboards, tasks, buckets, members, etc., belong to an organization.
"""

from influxdb_client import OrganizationsService, UsersService, Organization, PatchOrganizationRequest


class OrganizationsApi(object):
    """Implementation for '/api/v2/orgs' endpoint."""

    def __init__(self, influxdb_client):
        """Initialize defaults."""
        self._influxdb_client = influxdb_client
        self._organizations_service = OrganizationsService(influxdb_client.api_client)
        self._users_service = UsersService(influxdb_client.api_client)

    def me(self):
        """Return the current authenticated user."""
        user = self._users_service.get_me()
        return user

    def find_organization(self, org_id):
        """Retrieve an organization."""
        return self._organizations_service.get_orgs_id(org_id=org_id)

    def find_organizations(self, **kwargs):
        """
        List all organizations.

        :key int offset: Offset for pagination
        :key int limit: Limit for pagination
        :key bool descending:
        :key str org: Filter organizations to a specific organization name.
        :key str org_id: Filter organizations to a specific organization ID.
        :key str user_id: Filter organizations to a specific user ID.
        """
        return self._organizations_service.get_orgs(**kwargs).orgs

    def create_organization(self, name: str = None, organization: Organization = None) -> Organization:
        """Create an organization."""
        if organization is None:
            organization = Organization(name=name)
        return self._organizations_service.post_orgs(post_organization_request=organization)

    def update_organization(self, organization: Organization) -> Organization:
        """Update an organization.

        :param organization: Organization update to apply (required)
        :return: Organization
        """
        request = PatchOrganizationRequest(name=organization.name,
                                           description=organization.description)

        return self._organizations_service.patch_orgs_id(org_id=organization.id, patch_organization_request=request)

    def delete_organization(self, org_id: str):
        """Delete an organization."""
        return self._organizations_service.delete_orgs_id(org_id=org_id)



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from influxdb_client.service._base_service import _BaseService


class NotificationEndpointsService(_BaseService):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):  # noqa: E501,D401,D403
        """NotificationEndpointsService - a operation defined in OpenAPI."""
        super().__init__(api_client)

    def create_notification_endpoint(self, post_notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        """Add a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_notification_endpoint(post_notification_endpoint, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PostNotificationEndpoint post_notification_endpoint: Notification endpoint to create (required)
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_notification_endpoint_with_http_info(post_notification_endpoint, **kwargs)  # noqa: E501
        else:
            (data) = self.create_notification_endpoint_with_http_info(post_notification_endpoint, **kwargs)  # noqa: E501
            return data

    def create_notification_endpoint_with_http_info(self, post_notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        """Add a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_notification_endpoint_with_http_info(post_notification_endpoint, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PostNotificationEndpoint post_notification_endpoint: Notification endpoint to create (required)
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._create_notification_endpoint_prepare(post_notification_endpoint, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def create_notification_endpoint_async(self, post_notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        """Add a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param PostNotificationEndpoint post_notification_endpoint: Notification endpoint to create (required)
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._create_notification_endpoint_prepare(post_notification_endpoint, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _create_notification_endpoint_prepare(self, post_notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['post_notification_endpoint']  # noqa: E501
        self._check_operation_params('create_notification_endpoint', all_params, local_var_params)
        # verify the required parameter 'post_notification_endpoint' is set
        if ('post_notification_endpoint' not in local_var_params or
                local_var_params['post_notification_endpoint'] is None):
            raise ValueError("Missing the required parameter `post_notification_endpoint` when calling `create_notification_endpoint`")  # noqa: E501

        path_params = {}

        query_params = []

        header_params = {}

        body_params = None
        if 'post_notification_endpoint' in local_var_params:
            body_params = local_var_params['post_notification_endpoint']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def delete_notification_endpoints_id(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_notification_endpoints_id(endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_notification_endpoints_id_with_http_info(endpoint_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_notification_endpoints_id_with_http_info(endpoint_id, **kwargs)  # noqa: E501
            return data

    def delete_notification_endpoints_id_with_http_info(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_notification_endpoints_id_with_http_info(endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_notification_endpoints_id_prepare(endpoint_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def delete_notification_endpoints_id_async(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_notification_endpoints_id_prepare(endpoint_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _delete_notification_endpoints_id_prepare(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['endpoint_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('delete_notification_endpoints_id', all_params, local_var_params)
        # verify the required parameter 'endpoint_id' is set
        if ('endpoint_id' not in local_var_params or
                local_var_params['endpoint_id'] is None):
            raise ValueError("Missing the required parameter `endpoint_id` when calling `delete_notification_endpoints_id`")  # noqa: E501

        path_params = {}
        if 'endpoint_id' in local_var_params:
            path_params['endpointID'] = local_var_params['endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def delete_notification_endpoints_id_labels_id(self, endpoint_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a label from a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_notification_endpoints_id_labels_id(endpoint_id, label_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str label_id: The ID of the label to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_notification_endpoints_id_labels_id_with_http_info(endpoint_id, label_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_notification_endpoints_id_labels_id_with_http_info(endpoint_id, label_id, **kwargs)  # noqa: E501
            return data

    def delete_notification_endpoints_id_labels_id_with_http_info(self, endpoint_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a label from a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_notification_endpoints_id_labels_id_with_http_info(endpoint_id, label_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str label_id: The ID of the label to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_notification_endpoints_id_labels_id_prepare(endpoint_id, label_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}/labels/{labelID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def delete_notification_endpoints_id_labels_id_async(self, endpoint_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a label from a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str label_id: The ID of the label to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_notification_endpoints_id_labels_id_prepare(endpoint_id, label_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}/labels/{labelID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _delete_notification_endpoints_id_labels_id_prepare(self, endpoint_id, label_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['endpoint_id', 'label_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('delete_notification_endpoints_id_labels_id', all_params, local_var_params)
        # verify the required parameter 'endpoint_id' is set
        if ('endpoint_id' not in local_var_params or
                local_var_params['endpoint_id'] is None):
            raise ValueError("Missing the required parameter `endpoint_id` when calling `delete_notification_endpoints_id_labels_id`")  # noqa: E501
        # verify the required parameter 'label_id' is set
        if ('label_id' not in local_var_params or
                local_var_params['label_id'] is None):
            raise ValueError("Missing the required parameter `label_id` when calling `delete_notification_endpoints_id_labels_id`")  # noqa: E501

        path_params = {}
        if 'endpoint_id' in local_var_params:
            path_params['endpointID'] = local_var_params['endpoint_id']  # noqa: E501
        if 'label_id' in local_var_params:
            path_params['labelID'] = local_var_params['label_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_notification_endpoints(self, org_id, **kwargs):  # noqa: E501,D401,D403
        """List all notification endpoints.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_notification_endpoints(org_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str org_id: Only show notification endpoints that belong to specific organization ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :param int offset: The offset for pagination. The number of records to skip.  For more information about pagination parameters, see [Pagination](https://docs.influxdata.com/influxdb/latest/api/#tag/Pagination).
        :param int limit: Limits the number of records returned. Default is `20`.
        :return: NotificationEndpoints
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_notification_endpoints_with_http_info(org_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_notification_endpoints_with_http_info(org_id, **kwargs)  # noqa: E501
            return data

    def get_notification_endpoints_with_http_info(self, org_id, **kwargs):  # noqa: E501,D401,D403
        """List all notification endpoints.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_notification_endpoints_with_http_info(org_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str org_id: Only show notification endpoints that belong to specific organization ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :param int offset: The offset for pagination. The number of records to skip.  For more information about pagination parameters, see [Pagination](https://docs.influxdata.com/influxdb/latest/api/#tag/Pagination).
        :param int limit: Limits the number of records returned. Default is `20`.
        :return: NotificationEndpoints
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_notification_endpoints_prepare(org_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoints',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_notification_endpoints_async(self, org_id, **kwargs):  # noqa: E501,D401,D403
        """List all notification endpoints.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str org_id: Only show notification endpoints that belong to specific organization ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :param int offset: The offset for pagination. The number of records to skip.  For more information about pagination parameters, see [Pagination](https://docs.influxdata.com/influxdb/latest/api/#tag/Pagination).
        :param int limit: Limits the number of records returned. Default is `20`.
        :return: NotificationEndpoints
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_notification_endpoints_prepare(org_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoints',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_notification_endpoints_prepare(self, org_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['org_id', 'zap_trace_span', 'offset', 'limit']  # noqa: E501
        self._check_operation_params('get_notification_endpoints', all_params, local_var_params)
        # verify the required parameter 'org_id' is set
        if ('org_id' not in local_var_params or
                local_var_params['org_id'] is None):
            raise ValueError("Missing the required parameter `org_id` when calling `get_notification_endpoints`")  # noqa: E501

        if 'offset' in local_var_params and local_var_params['offset'] < 0:  # noqa: E501
            raise ValueError("Invalid value for parameter `offset` when calling `get_notification_endpoints`, must be a value greater than or equal to `0`")  # noqa: E501
        if 'limit' in local_var_params and local_var_params['limit'] > 100:  # noqa: E501
            raise ValueError("Invalid value for parameter `limit` when calling `get_notification_endpoints`, must be a value less than or equal to `100`")  # noqa: E501
        if 'limit' in local_var_params and local_var_params['limit'] < 1:  # noqa: E501
            raise ValueError("Invalid value for parameter `limit` when calling `get_notification_endpoints`, must be a value greater than or equal to `1`")  # noqa: E501
        path_params = {}

        query_params = []
        if 'offset' in local_var_params:
            query_params.append(('offset', local_var_params['offset']))  # noqa: E501
        if 'limit' in local_var_params:
            query_params.append(('limit', local_var_params['limit']))  # noqa: E501
        if 'org_id' in local_var_params:
            query_params.append(('orgID', local_var_params['org_id']))  # noqa: E501

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_notification_endpoints_id(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_notification_endpoints_id(endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_notification_endpoints_id_with_http_info(endpoint_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_notification_endpoints_id_with_http_info(endpoint_id, **kwargs)  # noqa: E501
            return data

    def get_notification_endpoints_id_with_http_info(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_notification_endpoints_id_with_http_info(endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_notification_endpoints_id_prepare(endpoint_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_notification_endpoints_id_async(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_notification_endpoints_id_prepare(endpoint_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_notification_endpoints_id_prepare(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['endpoint_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('get_notification_endpoints_id', all_params, local_var_params)
        # verify the required parameter 'endpoint_id' is set
        if ('endpoint_id' not in local_var_params or
                local_var_params['endpoint_id'] is None):
            raise ValueError("Missing the required parameter `endpoint_id` when calling `get_notification_endpoints_id`")  # noqa: E501

        path_params = {}
        if 'endpoint_id' in local_var_params:
            path_params['endpointID'] = local_var_params['endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_notification_endpoints_id_labels(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_notification_endpoints_id_labels(endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_notification_endpoints_id_labels_with_http_info(endpoint_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_notification_endpoints_id_labels_with_http_info(endpoint_id, **kwargs)  # noqa: E501
            return data

    def get_notification_endpoints_id_labels_with_http_info(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_notification_endpoints_id_labels_with_http_info(endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_notification_endpoints_id_labels_prepare(endpoint_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}/labels', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelsResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_notification_endpoints_id_labels_async(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_notification_endpoints_id_labels_prepare(endpoint_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}/labels', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelsResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_notification_endpoints_id_labels_prepare(self, endpoint_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['endpoint_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('get_notification_endpoints_id_labels', all_params, local_var_params)
        # verify the required parameter 'endpoint_id' is set
        if ('endpoint_id' not in local_var_params or
                local_var_params['endpoint_id'] is None):
            raise ValueError("Missing the required parameter `endpoint_id` when calling `get_notification_endpoints_id_labels`")  # noqa: E501

        path_params = {}
        if 'endpoint_id' in local_var_params:
            path_params['endpointID'] = local_var_params['endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def patch_notification_endpoints_id(self, endpoint_id, notification_endpoint_update, **kwargs):  # noqa: E501,D401,D403
        """Update a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_notification_endpoints_id(endpoint_id, notification_endpoint_update, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param NotificationEndpointUpdate notification_endpoint_update: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.patch_notification_endpoints_id_with_http_info(endpoint_id, notification_endpoint_update, **kwargs)  # noqa: E501
        else:
            (data) = self.patch_notification_endpoints_id_with_http_info(endpoint_id, notification_endpoint_update, **kwargs)  # noqa: E501
            return data

    def patch_notification_endpoints_id_with_http_info(self, endpoint_id, notification_endpoint_update, **kwargs):  # noqa: E501,D401,D403
        """Update a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_notification_endpoints_id_with_http_info(endpoint_id, notification_endpoint_update, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param NotificationEndpointUpdate notification_endpoint_update: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._patch_notification_endpoints_id_prepare(endpoint_id, notification_endpoint_update, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def patch_notification_endpoints_id_async(self, endpoint_id, notification_endpoint_update, **kwargs):  # noqa: E501,D401,D403
        """Update a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param NotificationEndpointUpdate notification_endpoint_update: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._patch_notification_endpoints_id_prepare(endpoint_id, notification_endpoint_update, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _patch_notification_endpoints_id_prepare(self, endpoint_id, notification_endpoint_update, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['endpoint_id', 'notification_endpoint_update', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('patch_notification_endpoints_id', all_params, local_var_params)
        # verify the required parameter 'endpoint_id' is set
        if ('endpoint_id' not in local_var_params or
                local_var_params['endpoint_id'] is None):
            raise ValueError("Missing the required parameter `endpoint_id` when calling `patch_notification_endpoints_id`")  # noqa: E501
        # verify the required parameter 'notification_endpoint_update' is set
        if ('notification_endpoint_update' not in local_var_params or
                local_var_params['notification_endpoint_update'] is None):
            raise ValueError("Missing the required parameter `notification_endpoint_update` when calling `patch_notification_endpoints_id`")  # noqa: E501

        path_params = {}
        if 'endpoint_id' in local_var_params:
            path_params['endpointID'] = local_var_params['endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'notification_endpoint_update' in local_var_params:
            body_params = local_var_params['notification_endpoint_update']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_notification_endpoint_id_labels(self, endpoint_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_notification_endpoint_id_labels(endpoint_id, label_mapping, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_notification_endpoint_id_labels_with_http_info(endpoint_id, label_mapping, **kwargs)  # noqa: E501
        else:
            (data) = self.post_notification_endpoint_id_labels_with_http_info(endpoint_id, label_mapping, **kwargs)  # noqa: E501
            return data

    def post_notification_endpoint_id_labels_with_http_info(self, endpoint_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_notification_endpoint_id_labels_with_http_info(endpoint_id, label_mapping, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_notification_endpoint_id_labels_prepare(endpoint_id, label_mapping, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}/labels', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_notification_endpoint_id_labels_async(self, endpoint_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_notification_endpoint_id_labels_prepare(endpoint_id, label_mapping, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}/labels', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_notification_endpoint_id_labels_prepare(self, endpoint_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['endpoint_id', 'label_mapping', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('post_notification_endpoint_id_labels', all_params, local_var_params)
        # verify the required parameter 'endpoint_id' is set
        if ('endpoint_id' not in local_var_params or
                local_var_params['endpoint_id'] is None):
            raise ValueError("Missing the required parameter `endpoint_id` when calling `post_notification_endpoint_id_labels`")  # noqa: E501
        # verify the required parameter 'label_mapping' is set
        if ('label_mapping' not in local_var_params or
                local_var_params['label_mapping'] is None):
            raise ValueError("Missing the required parameter `label_mapping` when calling `post_notification_endpoint_id_labels`")  # noqa: E501

        path_params = {}
        if 'endpoint_id' in local_var_params:
            path_params['endpointID'] = local_var_params['endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'label_mapping' in local_var_params:
            body_params = local_var_params['label_mapping']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def put_notification_endpoints_id(self, endpoint_id, notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        """Update a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_notification_endpoints_id(endpoint_id, notification_endpoint, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param NotificationEndpoint notification_endpoint: A new notification endpoint to replace the existing endpoint with (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.put_notification_endpoints_id_with_http_info(endpoint_id, notification_endpoint, **kwargs)  # noqa: E501
        else:
            (data) = self.put_notification_endpoints_id_with_http_info(endpoint_id, notification_endpoint, **kwargs)  # noqa: E501
            return data

    def put_notification_endpoints_id_with_http_info(self, endpoint_id, notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        """Update a notification endpoint.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_notification_endpoints_id_with_http_info(endpoint_id, notification_endpoint, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param NotificationEndpoint notification_endpoint: A new notification endpoint to replace the existing endpoint with (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._put_notification_endpoints_id_prepare(endpoint_id, notification_endpoint, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def put_notification_endpoints_id_async(self, endpoint_id, notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        """Update a notification endpoint.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str endpoint_id: The notification endpoint ID. (required)
        :param NotificationEndpoint notification_endpoint: A new notification endpoint to replace the existing endpoint with (required)
        :param str zap_trace_span: OpenTracing span context
        :return: NotificationEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._put_notification_endpoints_id_prepare(endpoint_id, notification_endpoint, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/notificationEndpoints/{endpointID}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='NotificationEndpoint',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _put_notification_endpoints_id_prepare(self, endpoint_id, notification_endpoint, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['endpoint_id', 'notification_endpoint', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('put_notification_endpoints_id', all_params, local_var_params)
        # verify the required parameter 'endpoint_id' is set
        if ('endpoint_id' not in local_var_params or
                local_var_params['endpoint_id'] is None):
            raise ValueError("Missing the required parameter `endpoint_id` when calling `put_notification_endpoints_id`")  # noqa: E501
        # verify the required parameter 'notification_endpoint' is set
        if ('notification_endpoint' not in local_var_params or
                local_var_params['notification_endpoint'] is None):
            raise ValueError("Missing the required parameter `notification_endpoint` when calling `put_notification_endpoints_id`")  # noqa: E501

        path_params = {}
        if 'endpoint_id' in local_var_params:
            path_params['endpointID'] = local_var_params['endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'notification_endpoint' in local_var_params:
            body_params = local_var_params['notification_endpoint']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class ScraperTargetRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'type': 'str',
        'url': 'str',
        'org_id': 'str',
        'bucket_id': 'str',
        'allow_insecure': 'bool'
    }

    attribute_map = {
        'name': 'name',
        'type': 'type',
        'url': 'url',
        'org_id': 'orgID',
        'bucket_id': 'bucketID',
        'allow_insecure': 'allowInsecure'
    }

    def __init__(self, name=None, type=None, url=None, org_id=None, bucket_id=None, allow_insecure=False):  # noqa: E501,D401,D403
        """ScraperTargetRequest - a model defined in OpenAPI."""  # noqa: E501
        self._name = None
        self._type = None
        self._url = None
        self._org_id = None
        self._bucket_id = None
        self._allow_insecure = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if type is not None:
            self.type = type
        if url is not None:
            self.url = url
        if org_id is not None:
            self.org_id = org_id
        if bucket_id is not None:
            self.bucket_id = bucket_id
        if allow_insecure is not None:
            self.allow_insecure = allow_insecure

    @property
    def name(self):
        """Get the name of this ScraperTargetRequest.

        The name of the scraper target.

        :return: The name of this ScraperTargetRequest.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this ScraperTargetRequest.

        The name of the scraper target.

        :param name: The name of this ScraperTargetRequest.
        :type: str
        """  # noqa: E501
        self._name = name

    @property
    def type(self):
        """Get the type of this ScraperTargetRequest.

        The type of the metrics to be parsed.

        :return: The type of this ScraperTargetRequest.
        :rtype: str
        """  # noqa: E501
        return self._type

    @type.setter
    def type(self, type):
        """Set the type of this ScraperTargetRequest.

        The type of the metrics to be parsed.

        :param type: The type of this ScraperTargetRequest.
        :type: str
        """  # noqa: E501
        self._type = type

    @property
    def url(self):
        """Get the url of this ScraperTargetRequest.

        The URL of the metrics endpoint.

        :return: The url of this ScraperTargetRequest.
        :rtype: str
        """  # noqa: E501
        return self._url

    @url.setter
    def url(self, url):
        """Set the url of this ScraperTargetRequest.

        The URL of the metrics endpoint.

        :param url: The url of this ScraperTargetRequest.
        :type: str
        """  # noqa: E501
        self._url = url

    @property
    def org_id(self):
        """Get the org_id of this ScraperTargetRequest.

        The organization ID.

        :return: The org_id of this ScraperTargetRequest.
        :rtype: str
        """  # noqa: E501
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Set the org_id of this ScraperTargetRequest.

        The organization ID.

        :param org_id: The org_id of this ScraperTargetRequest.
        :type: str
        """  # noqa: E501
        self._org_id = org_id

    @property
    def bucket_id(self):
        """Get the bucket_id of this ScraperTargetRequest.

        The ID of the bucket to write to.

        :return: The bucket_id of this ScraperTargetRequest.
        :rtype: str
        """  # noqa: E501
        return self._bucket_id

    @bucket_id.setter
    def bucket_id(self, bucket_id):
        """Set the bucket_id of this ScraperTargetRequest.

        The ID of the bucket to write to.

        :param bucket_id: The bucket_id of this ScraperTargetRequest.
        :type: str
        """  # noqa: E501
        self._bucket_id = bucket_id

    @property
    def allow_insecure(self):
        """Get the allow_insecure of this ScraperTargetRequest.

        Skip TLS verification on endpoint.

        :return: The allow_insecure of this ScraperTargetRequest.
        :rtype: bool
        """  # noqa: E501
        return self._allow_insecure

    @allow_insecure.setter
    def allow_insecure(self, allow_insecure):
        """Set the allow_insecure of this ScraperTargetRequest.

        Skip TLS verification on endpoint.

        :param allow_insecure: The allow_insecure of this ScraperTargetRequest.
        :type: bool
        """  # noqa: E501
        self._allow_insecure = allow_insecure

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, ScraperTargetRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



import itertools
import time
import types

from influxdb_client import WritePrecision
from influxdb_client.client.write_api import SYNCHRONOUS
from tests.base_test import BaseTest


class QueryStreamApi(BaseTest):

    def setUp(self) -> None:
        super().setUp()
        self.write_client = self.client.write_api(write_options=SYNCHRONOUS)
        self.bucket = self.create_test_bucket()

    def tearDown(self) -> None:
        self.write_client.close()
        super().tearDown()

    def test_block(self):
        self._prepareData()

        _result = self.query_api.query(
            f'from(bucket:"{self.bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)', self.org)

        self.assertEqual(len(_result), 1)
        self.assertEqual(len(_result[0].records), 100)

    def test_stream(self):
        self._prepareData()

        _result = self.query_api.query_stream(
            f'from(bucket:"{self.bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)', self.org)

        self.assertTrue(isinstance(_result, types.GeneratorType))
        _result_list = list(_result)

        self.assertEqual(len(_result_list), 100)

    def test_stream_break(self):
        self._prepareData()

        _result = self.query_api.query_stream(
            f'from(bucket:"{self.bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)', self.org)

        _result_list = list(itertools.islice(_result, 10))
        _result.close()

        self.assertEqual(len(_result_list), 10)

    def _prepareData(self):
        _list = [f'h2o_feet,location=coyote_creek water_level={x} {x}' for x in range(1, 101)]
        self.write_client.write(self.bucket.name, self.org, _list, write_precision=WritePrecision.S)
        time.sleep(1)
